@@ -41,6 +41,47 @@
 #define MM_PER_INCH     (25.4)
 #define PICA_PER_INCH   (6.0)
 
+/**
+ * rsvg_css_parse_vbox
+ * @vbox: The CSS viewBox
+ * @x : The X output
+ * @y: The Y output
+ * @w: The Width output
+ * @h: The Height output
+ *
+ * Returns: Success or failure
+ */
+gboolean
+rsvg_css_parse_vbox (const char * vbox, double * x, double * y,
+					 double * w, double * h)
+{
+	/* TODO: make me cleaner and more efficient */
+	char *ptr, *tok;
+	char *str = g_strdup (vbox);
+	gboolean has_vbox = FALSE;
+	
+	tok = strtok_r (str, ", \t", &ptr);
+	if (tok != NULL) {
+		*x = g_ascii_strtod (tok, NULL);
+		tok = strtok_r (NULL, ", \t", &ptr);
+		if (tok != NULL) {
+			*y = g_ascii_strtod (tok, NULL);
+			tok = strtok_r (NULL, ", \t", &ptr);
+			if (tok != NULL) {
+				*w = g_ascii_strtod (tok, NULL);
+				tok = strtok_r (NULL, ", \t", &ptr);
+				if (tok != NULL) {
+					*h = g_ascii_strtod (tok, NULL);
+					has_vbox = TRUE;
+				}
+			}
+		}
+	}
+	g_free (str);
+	
+	return has_vbox;
+}
+
 /**
  * rsvg_css_parse_length: Parse CSS2 length to a pixel value.
  * @str: Original string.
@@ -111,8 +152,7 @@ rsvg_css_parse_length (const char *str, gdouble pixels_per_inch,
  */
 double
 rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
-								 gdouble width_or_height, gdouble font_size,
-								 gdouble x_height)
+								 gdouble width_or_height, gdouble font_size)
 {
 	double length;
 	gint percent, em, ex;
@@ -124,7 +164,7 @@ rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
 	else if (em)
 		return length * font_size;
 	else if (ex)
-		return length * x_height;
+		return length * font_size * 2.0; /* stolen from imagemagick */
 	else
 		return length;
 }
