@@ -24,6 +24,7 @@
 
 #include "config.h"
 #include "rsvg-paint-server.h"
+#include "rsvg-private.h"
 
 #include <glib/gmem.h>
 #include <glib/gmessages.h>
@@ -151,7 +152,6 @@ rsvg_paint_server_lin_grad_render (RsvgPaintServer *self, ArtRender *ar,
 		{
 			if (rlg->stops->n_stop == 0)
 				{
-					/* g_warning ("gradient with no stops -- should be rejected by parser"); */
 					return;
 				}
 			agl = g_new (ArtGradientLinear, 1);
@@ -220,7 +220,6 @@ rsvg_paint_server_rad_grad_render (RsvgPaintServer *self, ArtRender *ar,
 		{
 			if (rrg->stops->n_stop == 0)
 				{
-					/* g_warning ("gradient with no stops -- should be rejected by parser"); */
 					return;
 				}
 			agr = g_new (ArtGradientRadial, 1);
@@ -236,6 +235,8 @@ rsvg_paint_server_rad_grad_render (RsvgPaintServer *self, ArtRender *ar,
 	art_affine_multiply (aff1, aff1, rrg->affine);
 	art_affine_invert (agr->affine, aff1);
 	
+	/* TODO: libart doesn't support spreads on radial gradients */
+
 	agr->fx = (rrg->fx - rrg->cx) / rrg->r;
 	agr->fy = (rrg->fy - rrg->cy) / rrg->r;
 	
@@ -353,57 +354,90 @@ rsvg_paint_server_unref (RsvgPaintServer *ps)
 }
 
 RsvgRadialGradient *
-rsvg_clone_radial_gradient (const RsvgRadialGradient *grad)
+rsvg_clone_radial_gradient (const RsvgRadialGradient *grad, gboolean * shallow_cloned)
 {
 	RsvgRadialGradient * clone = NULL;
 	int i;
 	
-	clone = g_new (RsvgRadialGradient, 1);
-	clone->super.type = grad->super.type;
-	clone->super.free = grad->super.free;
+	clone = g_new0 (RsvgRadialGradient, 1);
+	clone->super.type = RSVG_DEF_RADGRAD;
+	clone->super.free = rsvg_radial_gradient_free;
 	
 	for (i = 0; i < 6; i++)
 		clone->affine[i] = grad->affine[i];
-	clone->cx = grad->cx;
-	clone->cy = grad->cy;
-	clone->r  = grad->r;
-	clone->fx = grad->fx;
-	clone->fy = grad->fy;
+
+	if (grad->stops != NULL) {
+		clone->stops = g_new (RsvgGradientStops, 1);
+		clone->stops->n_stop = grad->stops->n_stop;
+		clone->stops->stop = g_new (RsvgGradientStop, grad->stops->n_stop);
 	
-	clone->stops = g_new (RsvgGradientStops, 1);
-	clone->stops->n_stop = grad->stops->n_stop;
-	clone->stops->stop = g_new (RsvgGradientStop, clone->stops->n_stop);
-	
-	for (i = 0; i < grad->stops->n_stop; i++)
-		clone->stops->stop[i] = grad->stops->stop[i];
+		for (i = 0; i < grad->stops->n_stop; i++)
+			clone->stops->stop[i] = grad->stops->stop[i];
+	} else {
+		clone->stops = NULL;
+	}
+
+	clone->spread = grad->spread;
+
+	/* EVIL EVIL - sodipodi can base LinearGradients on
+	   RadialGradients, and vice-versa. it is legal, though:
+	   http://www.w3.org/TR/SVG11/pservers.html#LinearGradients
+	*/
+	if (grad->super.type == RSVG_DEF_RADGRAD) {
+		clone->cx = grad->cx;
+		clone->cy = grad->cy;
+		clone->r  = grad->r;
+		clone->fx = grad->fx;
+		clone->fy = grad->fy;
+		
+		*shallow_cloned = FALSE;
+	} else {
+		*shallow_cloned = TRUE;
+	}
 	
 	return clone;
 }
 
 RsvgLinearGradient *
-rsvg_clone_linear_gradient (const RsvgLinearGradient *grad)
+rsvg_clone_linear_gradient (const RsvgLinearGradient *grad, gboolean * shallow_cloned)
 {
 	RsvgLinearGradient * clone = NULL;
 	int i;
 	
-	clone = g_new (RsvgLinearGradient, 1);
-	clone->super.type = grad->super.type;
-	clone->super.free = grad->super.free;
+	clone = g_new0 (RsvgLinearGradient, 1);
+	clone->super.type = RSVG_DEF_LINGRAD;
+	clone->super.free = rsvg_linear_gradient_free;
 	
 	for (i = 0; i < 6; i++)
 		clone->affine[i] = grad->affine[i];
-	clone->x1 = grad->x1;
-	clone->y1 = grad->y1;
-	clone->x2 = grad->x2;
-	clone->y2 = grad->y2;
+
+	if (grad->stops != NULL) {
+		clone->stops = g_new (RsvgGradientStops, 1);
+		clone->stops->n_stop = grad->stops->n_stop;
+		clone->stops->stop = g_new (RsvgGradientStop, grad->stops->n_stop);
+		
+		for (i = 0; i < grad->stops->n_stop; i++)
+			clone->stops->stop[i] = grad->stops->stop[i];
+	} else {
+		clone->stops = NULL;
+	}
+
 	clone->spread = grad->spread;
-	
-	clone->stops = g_new (RsvgGradientStops, 1);
-	clone->stops->n_stop = grad->stops->n_stop;
-	clone->stops->stop = g_new (RsvgGradientStop, clone->stops->n_stop);
-	
-	for (i = 0; i < grad->stops->n_stop; i++)
-		clone->stops->stop[i] = grad->stops->stop[i];
-	
+
+	/* EVIL EVIL - sodipodi can base LinearGradients on
+	   RadialGradients, and vice-versa. it is legal, though:
+	   http://www.w3.org/TR/SVG11/pservers.html#LinearGradients
+	*/
+	if (grad->super.type == RSVG_DEF_LINGRAD) {
+		clone->x1 = grad->x1;
+		clone->y1 = grad->y1;
+		clone->x2 = grad->x2;
+		clone->y2 = grad->y2;
+
+		*shallow_cloned = FALSE;
+	} else {
+		*shallow_cloned = TRUE;
+	}
+
 	return clone;
 }
