@@ -142,7 +142,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	else if (rsvg_css_param_match (str, "stroke-width"))
 		{
 			state->stroke_width = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
-																	(gdouble)ctx->height, state->font_size, 0.);
+																	(gdouble)ctx->height, state->font_size);
 		}
 	else if (rsvg_css_param_match (str, "stroke-linecap"))
 		{
@@ -173,7 +173,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	else if (rsvg_css_param_match (str, "font-size"))
 		{
 			state->font_size = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
-																 (gdouble)ctx->height, state->font_size, 0.);
+																 (gdouble)ctx->height, state->font_size);
 		}
 	else if (rsvg_css_param_match (str, "font-family"))
 		{
@@ -226,7 +226,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	else if (rsvg_css_param_match (str, "stroke-dashoffset"))
 		{
 			state->dash.offset = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
-																   rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size, 0.);
+																   rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
 			if (state->dash.offset < 0.)
 				state->dash.offset = 0.;
 		}
@@ -413,7 +413,7 @@ rsvg_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
 /* Parse an SVG transform string into an affine matrix. Reference: SVG
    working draft dated 1999-07-06, section 8.5. Return TRUE on
    success. */
-static gboolean
+gboolean
 rsvg_parse_transform (double dst[6], const char *src)
 {
 	int idx;
@@ -591,6 +591,7 @@ void
 rsvg_parse_style_attrs (RsvgHandle *ctx, 
 						const char * tag,
 						const char * klazz,
+						const char * id,
 						const xmlChar **atts)
 {
 	int i = 0, j = 0;
@@ -598,6 +599,18 @@ rsvg_parse_style_attrs (RsvgHandle *ctx,
 	gboolean found = FALSE;
 	GString * klazz_list = NULL;
 	
+	/* handle the all-encompassing "star" entry first, ignoring found-ness */
+	rsvg_lookup_apply_css_style (ctx, "*");
+
+	if (id != NULL)
+		{
+			target = g_strdup_printf ("#%s", id);
+			rsvg_lookup_apply_css_style (ctx, target);
+			g_free (target);
+		}
+
+	/* todo: see if klazz or tag should accumulate onto tag.klazz */
+
 	if (tag != NULL && klazz != NULL)
 		{
 			target = g_strdup_printf ("%s.%s", tag, klazz);
