@@ -211,7 +211,7 @@ static void
 rsvg_start_g (RsvgHandle *ctx, const xmlChar **atts)
 {
 	RsvgState *state = &ctx->state[ctx->n_state - 1];
-	const char * klazz = NULL;
+	const char * klazz = NULL, * id = NULL;
 	int i;
 	
 	if (atts != NULL)
@@ -220,10 +220,12 @@ rsvg_start_g (RsvgHandle *ctx, const xmlChar **atts)
 				{
 					if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
-	rsvg_parse_style_attrs (ctx, "g", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "g", klazz, id, atts);
 	if (state->opacity != 0xff)
 		rsvg_push_opacity_group (ctx);
 }
@@ -380,7 +382,8 @@ rsvg_gradient_stop_handler_new (RsvgHandle *ctx, RsvgGradientStops **p_stops,
 	return &gstops->super;
 }
 
-static void
+/* exported to the paint server via rsvg-private.h */
+void
 rsvg_linear_gradient_free (RsvgDefVal *self)
 {
 	RsvgLinearGradient *z = (RsvgLinearGradient *)self;
@@ -396,13 +399,14 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, const xmlChar **atts)
 	RsvgState *state = &ctx->state[ctx->n_state - 1];
 	RsvgLinearGradient *grad = NULL;
 	int i;
-	char *id = NULL;
+	const char *id = NULL;
 	double x1 = 0., y1 = 0., x2 = 0., y2 = 0.;
 	ArtGradientSpread spread = ART_GRADIENT_PAD;
 	const char * xlink_href = NULL;
-	gboolean got_x1, got_x2, got_y1, got_y2, got_spread, cloned;
-	
-	got_x1 = got_x2 = got_y1 = got_y2 = got_spread = cloned = FALSE;
+	gboolean got_x1, got_x2, got_y1, got_y2, got_spread, got_transform, cloned, shallow_cloned;
+	double affine[6];
+
+	got_x1 = got_x2 = got_y1 = got_y2 = got_spread = got_transform = cloned = shallow_cloned = FALSE;
 	
 	/* 100% is the default */
 	x2 = rsvg_css_parse_normalized_length ("100%", ctx->dpi, (gdouble)ctx->width, state->font_size);
@@ -413,26 +417,43 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, const xmlChar **atts)
 			for (i = 0; atts[i] != NULL; i += 2)
 				{
 					if (!strcmp ((char *)atts[i], "id"))
-						id = (char *)atts[i + 1];
-					else if (!strcmp ((char *)atts[i], "x1"))
+						id = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "x1")) {
 						x1 = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->width, state->font_size);
-					else if (!strcmp ((char *)atts[i], "y1"))
+						got_x1 = TRUE;
+					}
+					else if (!strcmp ((char *)atts[i], "y1")) {
 						y1 = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->height, state->font_size);
-					else if (!strcmp ((char *)atts[i], "x2"))
+						got_y1 = TRUE;
+					}
+					else if (!strcmp ((char *)atts[i], "x2")) {
 						x2 = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->width, state->font_size);
-					else if (!strcmp ((char *)atts[i], "y2"))
+						got_x2 = TRUE;
+					}
+					else if (!strcmp ((char *)atts[i], "y2")) {
 						y2 = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->height, state->font_size);
+						got_y2 = TRUE;
+					}
 					else if (!strcmp ((char *)atts[i], "spreadMethod"))
 						{
-							if (!strcmp ((char *)atts[i + 1], "pad"))
+							if (!strcmp ((char *)atts[i + 1], "pad")) {
 								spread = ART_GRADIENT_PAD;
-							else if (!strcmp ((char *)atts[i + 1], "reflect"))
+								got_spread = TRUE;
+							}
+							else if (!strcmp ((char *)atts[i + 1], "reflect")) {
 								spread = ART_GRADIENT_REFLECT;
-							else if (!strcmp ((char *)atts[i + 1], "repeat"))
+								got_spread = TRUE;
+							}
+							else if (!strcmp ((char *)atts[i + 1], "repeat")) {
 								spread = ART_GRADIENT_REPEAT;
+								got_spread = TRUE;
+							}
 						}
 					else if (!strcmp ((char *)atts[i], "xlink:href"))
 						xlink_href = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "gradientTransform")) {
+						got_transform = rsvg_parse_transform (affine, (const char *)atts[i + 1]);
+					}
 				}
 		}
 	
@@ -442,7 +463,7 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, const xmlChar **atts)
 			if (parent != NULL)
 				{
 					cloned = TRUE;
-					grad = rsvg_clone_linear_gradient (parent); 
+					grad = rsvg_clone_linear_gradient (parent, &shallow_cloned); 
 					ctx->handler = rsvg_gradient_stop_handler_new_clone (ctx, grad->stops, "linearGradient");
 				}
 		}
@@ -459,6 +480,9 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, const xmlChar **atts)
 	
 	for (i = 0; i < 6; i++)
 		grad->affine[i] = state->affine[i];
+
+	if (got_transform)
+		art_affine_multiply (grad->affine, affine, grad->affine);
 	
 	/* state inherits parent/cloned information unless it's explicity gotten */
 	grad->x1 = (cloned && !got_x1) ? grad->x1 : x1;
@@ -468,7 +492,8 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, const xmlChar **atts)
 	grad->spread = (cloned && !got_spread) ? grad->spread : spread;
 }
 
-static void
+/* exported to the paint server via rsvg-private.h */
+void
 rsvg_radial_gradient_free (RsvgDefVal *self)
 {
 	RsvgRadialGradient *z = (RsvgRadialGradient *)self;
@@ -479,17 +504,19 @@ rsvg_radial_gradient_free (RsvgDefVal *self)
 }
 
 static void
-rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts)
+rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts, const char * tag) /* tag for conicalGradient */
 {
 	RsvgState *state = &ctx->state[ctx->n_state - 1];
 	RsvgRadialGradient *grad = NULL;
 	int i;
-	char *id = NULL;
+	const char *id = NULL;
 	double cx = 0., cy = 0., r = 0., fx = 0., fy = 0.;  
 	const char * xlink_href = NULL;
-	gboolean got_cx, got_cy, got_r, got_fx, got_fy, cloned;
+	ArtGradientSpread spread = ART_GRADIENT_PAD;
+	gboolean got_cx, got_cy, got_r, got_fx, got_fy, got_spread, got_transform, cloned, shallow_cloned;
+	double affine[6];
 	
-	got_cx = got_cy = got_r = got_fx = got_fy = cloned = FALSE;
+	got_cx = got_cy = got_r = got_fx = got_fy = got_spread = got_transform = cloned = shallow_cloned = FALSE;
 	
 	/* setup defaults */
 	cx = rsvg_css_parse_normalized_length ("50%", ctx->dpi, (gdouble)ctx->width, state->font_size);
@@ -502,7 +529,7 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts)
 			for (i = 0; atts[i] != NULL; i += 2)
 				{
 					if (!strcmp ((char *)atts[i], "id"))
-						id = (char *)atts[i + 1];
+						id = (const char *)atts[i + 1];
 					else if (!strcmp ((char *)atts[i], "cx")) {
 						cx = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->width, state->font_size);
 						got_cx = TRUE;
@@ -527,6 +554,24 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts)
 					}
 					else if (!strcmp ((char *)atts[i], "xlink:href"))
 						xlink_href = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "gradientTransform")) {
+						got_transform = rsvg_parse_transform (affine, (const char *)atts[i + 1]);
+					}
+					else if (!strcmp ((char *)atts[i], "spreadMethod"))
+						{
+							if (!strcmp ((char *)atts[i + 1], "pad")) {
+								spread = ART_GRADIENT_PAD;
+								got_spread = TRUE;
+							}
+							else if (!strcmp ((char *)atts[i + 1], "reflect")) {
+								spread = ART_GRADIENT_REFLECT;
+								got_spread = TRUE;
+							}
+							else if (!strcmp ((char *)atts[i + 1], "repeat")) {
+								spread = ART_GRADIENT_REPEAT;
+								got_spread = TRUE;
+							}
+						}
 				}
 		}
 	
@@ -536,8 +581,8 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts)
 			if (parent != NULL)
 				{
 					cloned = TRUE;
-					grad = rsvg_clone_radial_gradient (parent); 
-					ctx->handler = rsvg_gradient_stop_handler_new_clone (ctx, grad->stops, "radialGradient");
+					grad = rsvg_clone_radial_gradient (parent, &shallow_cloned); 
+					ctx->handler = rsvg_gradient_stop_handler_new_clone (ctx, grad->stops, tag);
 				}
     }
 	if (!cloned)
@@ -545,25 +590,35 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, const xmlChar **atts)
 			grad = g_new (RsvgRadialGradient, 1);
 			grad->super.type = RSVG_DEF_RADGRAD;
 			grad->super.free = rsvg_radial_gradient_free;
-			ctx->handler = rsvg_gradient_stop_handler_new (ctx, &grad->stops, "radialGradient");
-			
-			if (!got_fx)
-				fx = cx;
-			if (!got_fy)
-				fy = cy;
+			ctx->handler = rsvg_gradient_stop_handler_new (ctx, &grad->stops, tag);		   
 		}
+
+	if (!cloned || shallow_cloned) {
+		if (!got_fx) {
+			fx = cx;
+			got_fx = TRUE;
+		}
+		if (!got_fy) {
+			fy = cy;
+			got_fy = TRUE;
+		}
+	}
 	
 	rsvg_defs_set (ctx->defs, id, &grad->super);
 	
 	for (i = 0; i < 6; i++)
 		grad->affine[i] = state->affine[i];
+
+	if (got_transform)
+		art_affine_multiply (grad->affine, affine, grad->affine);
 	
 	/* state inherits parent/cloned information unless it's explicity gotten */
-	grad->cx = (cloned && !got_cx) ? grad->cx: cx;
-	grad->cy = (cloned && !got_cy) ? grad->cy: cy;
-	grad->r =  (cloned && !got_r) ? grad->r : r;
+	grad->cx = (cloned && !got_cx) ? grad->cx : cx;
+	grad->cy = (cloned && !got_cy) ? grad->cy : cy;
+	grad->r =  (cloned && !got_r)  ? grad->r  : r;
 	grad->fx = (cloned && !got_fx) ? grad->fx : fx;
 	grad->fy = (cloned && !got_fy) ? grad->fy : fy;
+	grad->spread = (cloned && !got_spread) ? grad->spread : spread;
 }
 
 /* end gradients */
@@ -655,11 +710,19 @@ rsvg_defs_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 	else
 		rsvg_state_init (ctx->state);
 	ctx->n_state++;
-	
+
+	/**
+	 * conicalGradient isn't in the SVG spec and I'm not sure exactly what it does. libart definitely
+	 * has no analogue. But it does seem similar enough to a radialGradient that i'd rather get the
+	 * onscreen representation of the colour wrong than not have any colour displayed whatsoever
+	 */
+
 	if (!strcmp ((char *)name, "linearGradient"))
 		rsvg_start_linear_gradient (ctx, atts);
 	else if (!strcmp ((char *)name, "radialGradient"))
-		rsvg_start_radial_gradient (ctx, atts);
+		rsvg_start_radial_gradient (ctx, atts, "radialGradient");
+	else if (!strcmp((char *)name, "conicalGradient"))
+		rsvg_start_radial_gradient (ctx, atts, "conicalGradient");
 	else if (!strcmp ((char *)name, "style"))
 		rsvg_start_style (ctx, atts);
 }
@@ -748,11 +811,13 @@ rsvg_start_element (void *data, const xmlChar *name, const xmlChar **atts)
 			else if (!strcmp ((char *)name, "polyline"))
 				rsvg_start_polyline (ctx, atts);
 			
-			/* */
+			/* see conicalGradient discussion above */
 			else if (!strcmp ((char *)name, "linearGradient"))
 				rsvg_start_linear_gradient (ctx, atts);
 			else if (!strcmp ((char *)name, "radialGradient"))
-				rsvg_start_radial_gradient (ctx, atts);
+				rsvg_start_radial_gradient (ctx, atts, "radialGradient");
+			else if (!strcmp ((char *)name, "conicalGradient"))
+				rsvg_start_radial_gradient (ctx, atts, "conicalGradient");
     }
 }
 
